# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import StrictStr, Field, BaseModel, Field, StrictInt, StrictStr, conlist, constr 
from finbourne_access.models.link import Link
from finbourne_access.models.role_id import RoleId
from finbourne_access.models.role_resource_request import RoleResourceRequest
from finbourne_access.models.when_spec import WhenSpec

class RoleResponse(BaseModel):
    """
    Response object from the role API  # noqa: E501
    """
    id: RoleId = Field(...)
    role_hierarchy_index: StrictInt = Field(..., alias="roleHierarchyIndex", description="The hierarchical index of the role")
    description:  Optional[StrictStr] = Field(None,alias="description", description="The description of the role") 
    resource: RoleResourceRequest = Field(...)
    when: WhenSpec = Field(...)
    permission:  StrictStr = Field(...,alias="permission", description="The action key of the role") 
    limit: Optional[Dict[str, StrictStr]] = Field(None, description="The identifiers of the role with the maximum privileges that this role can have")
    links: Optional[conlist(Link)] = None
    __properties = ["id", "roleHierarchyIndex", "description", "resource", "when", "permission", "limit", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RoleResponse:
        """Create an instance of RoleResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of id
        if self.id:
            _dict['id'] = self.id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of resource
        if self.resource:
            _dict['resource'] = self.resource.to_dict()
        # override the default output from pydantic by calling `to_dict()` of when
        if self.when:
            _dict['when'] = self.when.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if limit (nullable) is None
        # and __fields_set__ contains the field
        if self.limit is None and "limit" in self.__fields_set__:
            _dict['limit'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RoleResponse:
        """Create an instance of RoleResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RoleResponse.parse_obj(obj)

        _obj = RoleResponse.parse_obj({
            "id": RoleId.from_dict(obj.get("id")) if obj.get("id") is not None else None,
            "role_hierarchy_index": obj.get("roleHierarchyIndex"),
            "description": obj.get("description"),
            "resource": RoleResourceRequest.from_dict(obj.get("resource")) if obj.get("resource") is not None else None,
            "when": WhenSpec.from_dict(obj.get("when")) if obj.get("when") is not None else None,
            "permission": obj.get("permission"),
            "limit": obj.get("limit"),
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj
