# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict
from pydantic.v1 import StrictStr, Field, BaseModel, Field 
from finbourne_access.models.request_details import RequestDetails
from finbourne_access.models.resource_details import ResourceDetails

class EvaluationRequest(BaseModel):
    """
    Specification for a server side evaluation of entitlement.  # noqa: E501
    """
    request: RequestDetails = Field(...)
    resource: ResourceDetails = Field(...)
    __properties = ["request", "resource"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EvaluationRequest:
        """Create an instance of EvaluationRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of request
        if self.request:
            _dict['request'] = self.request.to_dict()
        # override the default output from pydantic by calling `to_dict()` of resource
        if self.resource:
            _dict['resource'] = self.resource.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> EvaluationRequest:
        """Create an instance of EvaluationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return EvaluationRequest.parse_obj(obj)

        _obj = EvaluationRequest.parse_obj({
            "request": RequestDetails.from_dict(obj.get("request")) if obj.get("request") is not None else None,
            "resource": ResourceDetails.from_dict(obj.get("resource")) if obj.get("resource") is not None else None
        })
        return _obj
