# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class DateUnit(str, Enum):
    """
    DateUnit
    """

    """
    allowed enum values
    """
    UNDEFINED = 'Undefined'
    MINUTE = 'Minute'
    HOUR = 'Hour'
    DAY = 'Day'
    BUSINESSDAY = 'BusinessDay'
    WEEKS = 'Weeks'
    MONTHS = 'Months'
    YEARS = 'Years'

    @classmethod
    def from_json(cls, json_str: str) -> DateUnit:
        """Create an instance of DateUnit from a JSON string"""
        return DateUnit(json.loads(json_str))
